@@ -13,10 +13,10 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,6 +59,8 @@
 
 struct _XfceKiosk
 {
+  GObject __parent__;
+
   gchar  *module_name;
   XfceRc *module_rc;
 };
@@ -67,8 +69,8 @@ struct _XfceKiosk
 static const gchar *xfce_kiosk_lookup  (const XfceKiosk *kiosk,
                                         const gchar     *capability);
 static gboolean     xfce_kiosk_chkgrp  (const gchar     *group);
-static void         xfce_kiosk_init    (void);
 static time_t       mtime              (const gchar     *path);
+static void         xfce_kiosk_finalize (GObject        *object);
 
 
 static gchar        *usrname = NULL;
@@ -77,10 +79,19 @@ static time_t        kiosktime = 0;
 static const gchar  *kioskdef = NULL;
 static XfceRc       *kioskrc = NULL;
 
-
+G_DEFINE_TYPE (XfceKiosk, xfce_kiosk, G_TYPE_OBJECT)
 G_LOCK_DEFINE_STATIC (kiosk_lock);
 
 
+static void
+xfce_kiosk_class_init (XfceKioskClass *klass)
+{
+  GObjectClass *gobject_class;
+
+  gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->finalize = xfce_kiosk_finalize;
+}
+
 /**
  * xfce_kiosk_new:
  * @module:
@@ -98,11 +109,9 @@ xfce_kiosk_new (const gchar *module)
   g_return_val_if_fail (module != NULL, NULL);
   g_return_val_if_fail (strcmp (module, "General") != 0, NULL);
 
-  xfce_kiosk_init ();
-
   g_snprintf (path, 1024, "%s/%s.kioskrc", KIOSKDIR, module);
 
-  kiosk               = g_new (XfceKiosk, 1);
+  kiosk               = g_object_new (XFCE_TYPE_KIOSK, NULL);
   kiosk->module_name  = g_strdup (module);
   kiosk->module_rc    = xfce_rc_simple_open (path, TRUE);
 
@@ -179,6 +188,19 @@ xfce_kiosk_query (const XfceKiosk *kiosk,
 }
 
 
+static void
+xfce_kiosk_finalize (GObject *object)
+{
+  XfceKiosk *kiosk = XFCE_KIOSK (object);
+
+  g_return_if_fail (kiosk != NULL);
+
+  if (kiosk->module_rc != NULL)
+    xfce_rc_close (kiosk->module_rc);
+  g_free (kiosk->module_name);
+}
+
+
 /**
  * xfce_kiosk_free:
  * @kiosk: A #XfceKiosk.
@@ -190,12 +212,9 @@ xfce_kiosk_query (const XfceKiosk *kiosk,
 void
 xfce_kiosk_free (XfceKiosk *kiosk)
 {
-  g_return_if_fail (kiosk != NULL);
-
-  if (kiosk->module_rc != NULL)
-    xfce_rc_close (kiosk->module_rc);
-  g_free (kiosk->module_name);
-  g_free (kiosk);
+  /* finalize takes care of things in case the consumer calls unref
+   * themselves instead of this function */
+  g_object_unref (kiosk);
 }
 
 
@@ -243,7 +262,7 @@ xfce_kiosk_chkgrp (const gchar *group)
 
 
 static void
-xfce_kiosk_init (void)
+xfce_kiosk_init (XfceKiosk *kiosk)
 {
   struct passwd *pw;
   struct group  *gr;
@@ -338,7 +357,7 @@ mtime (const gchar *path)
   struct stat sb;
 
   if (G_UNLIKELY (path == NULL) || stat (path, &sb) < 0)
-    return (time_t) 0; 
+    return (time_t) 0;
 
   return sb.st_mtime;
 }
